package com.gic.haoban.manage.web.controller;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Comparator;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.date.DateUtil;

import com.alibaba.dubbo.common.utils.CollectionUtils;
import com.alibaba.fastjson.JSONObject;
import com.gic.clerk.api.dto.ClerkDTO;
import com.gic.clerk.api.service.ClerkService;
import com.gic.commons.util.EntityUtil;
import com.gic.enterprise.api.dto.StoreDTO;
import com.gic.enterprise.api.dto.StorePhotoDTO;
import com.gic.enterprise.api.service.StoreService;
import com.gic.haoban.common.utils.HaobanResponse;
import com.gic.haoban.data.api.dto.HaobanDataDTO;
import com.gic.haoban.data.api.service.HaobanDataApiService;
import com.gic.haoban.manage.api.dto.AuditSettingDTO;
import com.gic.haoban.manage.api.dto.DepartmentDTO;
import com.gic.haoban.manage.api.dto.EnterpriseDetailDTO;
import com.gic.haoban.manage.api.dto.StaffDTO;
import com.gic.haoban.manage.api.dto.StaffDepartmentRelatedDTO;
import com.gic.haoban.manage.api.enums.StoreFieldEnum;
import com.gic.haoban.manage.api.service.AuditApiService;
import com.gic.haoban.manage.api.service.AuditSettingApiService;
import com.gic.haoban.manage.api.service.DepartmentApiService;
import com.gic.haoban.manage.api.service.StaffApiService;
import com.gic.haoban.manage.api.service.StaffDepartmentRelatedApiService;
import com.gic.haoban.manage.api.service.WxEnterpriseRelatedApiService;
import com.gic.haoban.manage.web.errCode.HaoBanErrCode;
import com.gic.haoban.manage.web.vo.ClerkVo;
import com.gic.haoban.manage.web.vo.StoreDetailVO;
import com.gic.haoban.manage.web.vo.StoreVO;

@RestController
public class StoreController extends WebBaseController{
	@Autowired
	private ClerkService clerkService;
	@Autowired
	private StaffApiService staffApiService;
	@Autowired
	private DepartmentApiService departmentApiService;
	@Autowired
	private StaffDepartmentRelatedApiService staffDepartmentRelatedApiService;
	@Autowired
	private WxEnterpriseRelatedApiService wxEnterpriseRelatedApiService;
	@Autowired
	private HaobanDataApiService haobanDataApiService;
	@Autowired
	private StoreService storeService;
	@Autowired
	private AuditApiService auditApiService;
	@Autowired
	private AuditSettingApiService auditSettingApiService;
	//门店列表
	@RequestMapping("/store-list")
	public HaobanResponse storeList(String staffId,String wxEnterpriseId) {
		if(StringUtils.isAnyBlank(staffId,wxEnterpriseId)){
			return resultResponse(HaoBanErrCode.ERR_2);
		}
		List<StoreVO> resultList = new ArrayList<StoreVO>();
		//2、获取关联表数据
		List<StaffDepartmentRelatedDTO> relateList = staffDepartmentRelatedApiService.listByStaffId(staffId);
		if(CollectionUtil.isNotEmpty(relateList)){
			List<String> departmentIds = relateList.stream().map(s->s.getDepartmentId()).collect(Collectors.toList());
			List<DepartmentDTO> departmentList = departmentApiService.listByDepartmentIds(departmentIds, 1);
			//1、初始化Department表数据
			for(DepartmentDTO dto : departmentList){
				StoreVO VO = new StoreVO();
				VO.setDepartmentId(dto.getDepartmentId());
				VO.setStoreId(dto.getRelatedId());//为空，则无该值
				VO.setStoreName(dto.getDepartmentName());
				VO.setWxEnterpriseId(dto.getWxEnterpriseId());
				VO.setStoreImg("");//先置空
				if(org.apache.commons.lang.StringUtils.isNotBlank(dto.getRelatedId())){
					StoreDTO storeDTO = storeService.getStore(dto.getRelatedId());
					if(storeDTO != null ){
						VO.setEnterpriseId(storeDTO.getEnterpriseId());
					}
				}
				VO.setCreateTime(dto.getCreateTime());
				resultList.add(VO);
			}
			
			
			Map<String,StaffDepartmentRelatedDTO> map = com.gic.commons.util.CollectionUtil.toMap(relateList, "departmentId");
			for(StoreVO VO : resultList){
				StaffDepartmentRelatedDTO staffDepartmentRelatedDTO = map.get(VO.getDepartmentId());
				if(staffDepartmentRelatedDTO != null){
					String clerkCode = staffDepartmentRelatedDTO.getClerkCode();
					VO.setBindFlag(StringUtils.isEmpty(clerkCode)?0:1);
					VO.setClerkCode(clerkCode);
					VO.setStaffDepartmentRelatedId(staffDepartmentRelatedDTO.getStaffDepartmentRelatedId());
					VO.setStaffId(staffId);
					ClerkDTO clerkDTO  = clerkService.getClerkByClerkCode(VO.getEnterpriseId(), clerkCode);
					VO.setClerkId(clerkDTO == null?"":clerkDTO.getClerkId());
				}else{
					VO.setBindFlag(0);
				}
			}
		}
		//3、判断是否是店长
		for(StoreVO VO : resultList){
			//查gic门店
			StoreDTO storeDTO = storeService.getStore(VO.getStoreId());
			List<StorePhotoDTO> imgList = storeService.getStoreImages(VO.getStoreId());
			EnterpriseDetailDTO detail = wxEnterpriseRelatedApiService.getByEnterpriseId(storeDTO.getEnterpriseId());
			VO.setWxEnterpriseRelatedId(detail.getWxEnterpriseRelatedId());
			//已经绑定的门店
			if(VO.getBindFlag()==1){
				String clerkCode = VO.getClerkCode();
				if(StringUtils.isEmpty(clerkCode)){
					VO.setClerkType(0);
				}else{
					if(storeDTO != null){
						//查gic门店店长
						ClerkDTO clerkDTO = clerkService.getClerkByClerkId(storeDTO.getClerkId());
						if(clerkDTO != null){
							//code相同，则置为1
							if(clerkCode.equals(clerkDTO.getClerkCode())){
								VO.setClerkType(1);
							}
						}
					}
				}
			}
			if(CollectionUtil.isNotEmpty(imgList)){
				VO.setStoreImg(imgList.get(0).getImageUrl());;
			}
		}
		if(CollectionUtils.isNotEmpty(resultList)){
			Map<Integer,StoreVO> map = com.gic.commons.util.CollectionUtil.toMap(resultList, "clerkType");
			if(map.containsKey(1)){
				//店长
				resultList = resultList.stream().sorted(Comparator.comparing(StoreVO::getBindFlag).thenComparing(StoreVO::getClerkType,Comparator.reverseOrder()).thenComparing(StoreVO::getCreateTime,Comparator.reverseOrder())).collect(Collectors.toList());
			}else{
				//店员
				resultList = resultList.stream().sorted(Comparator.comparing(StoreVO::getBindFlag).thenComparing(StoreVO::getClerkType,Comparator.reverseOrder()).thenComparing(StoreVO::getCreateTime,Comparator.reverseOrder())).collect(Collectors.toList());
			}
			
			
		}
		return resultResponse(HaoBanErrCode.ERR_1,resultList);
	}
	//获取首页销售数据
	@RequestMapping("/sale-date")
	public HaobanResponse saleDate(String storeId,String clerkType,String clerkId) {
		if(StringUtils.isAnyBlank(storeId,clerkType,clerkId)){
			return resultResponse(HaoBanErrCode.ERR_2);
		}
		StoreDTO store = storeService.getStore(storeId);
		String enterpriseId = store.getEnterpriseId();
		String date = DateUtil.format(new Date(), "yyyy-MM-dd");
		if("0".equals(clerkType)){
			HaobanDataDTO HaobanDataDTO = haobanDataApiService.getSaleDataByDateByClerkId(enterpriseId, storeId,clerkId, date);
			return resultResponse(HaoBanErrCode.ERR_1,HaobanDataDTO);
		}
		HaobanDataDTO HaobanDataDTO = haobanDataApiService.getSaleDataByDate(enterpriseId, storeId, date);
		return resultResponse(HaoBanErrCode.ERR_1,HaobanDataDTO);
	}
	
	//获取首页销售数据
	@RequestMapping("/change-store-detail")
	public HaobanResponse saleDate(String wxEnterpriseId,String storeId,String staffId,String changeField,String oldValue,String newValue) {
		if(StringUtils.isAnyBlank(wxEnterpriseId,storeId,changeField,oldValue,newValue)){
			return resultResponse(HaoBanErrCode.ERR_2);
		}
		StoreDTO store = storeService.getStore(storeId);
		String enterpriseId = store.getEnterpriseId();
		AuditSettingDTO auditSettingDTO = auditSettingApiService.findSettingByWxEnterpriseId(wxEnterpriseId);
		//默认不审核
		int auditFlag = 0;
		if(auditSettingDTO != null){
			auditFlag = auditSettingDTO.getAuditFlag();
		}
		if(auditFlag == 1){
			//要审核
			int i = auditApiService.save(1, enterpriseId, staffId, storeId, changeField, oldValue, newValue);
		}else{
			//TODO 枚举变量的使用
		 if(!StoreFieldEnum.STORE_IMAG.getValue().equals(changeField)){
			 //非门店图片修改
			getStoreDTO( store, changeField, oldValue, newValue);
			storeService.saveOrUpdateStore(store);
		 }else{
			 //门店图片修改
			 List<StorePhotoDTO> oldStorePhoto = JSONObject.parseArray(oldValue, StorePhotoDTO.class);
			 List<StorePhotoDTO> newStorePhoto = JSONObject.parseArray(newValue, StorePhotoDTO.class);
			 List<String> delPhotos = new ArrayList<>();
//			 storeService.editStoreImage( storeId, newStorePhoto,  delPhotos);
		 }
		}
		return resultResponse(HaoBanErrCode.ERR_1);
	}
	void getStoreDTO(StoreDTO store,String changeField,String oldValue,String newValue){
		if(StoreFieldEnum.BUSINESS_TIME_STRING.getValue().equals(changeField)){
			SimpleDateFormat sdf = new SimpleDateFormat("HH:mm");
			//营业时间
			String [] newValues = newValue.split("-");
			String openTime = newValues[0];
			String closeTime = newValues[1];
			Date date1 = null;
			Date date2 = null;
			try {
				date1 = sdf.parse(openTime);
			} catch (ParseException e) {
				e.printStackTrace();
			}
			try {
				date2 = sdf.parse(closeTime);
			} catch (ParseException e) {
				e.printStackTrace();
			}
			store.setOpenTime(date1);
			store.setCloseTime(date2);
		}else if(StoreFieldEnum.CONACTS_PHONE.getValue().equals(changeField)){
			//联系电话
			store.setConactsPhone(newValue);
		}else if(StoreFieldEnum.STORE_ADDRESS.getValue().equals(changeField)){
			//门店地址
			store.setConactsPhone(newValue);
		}else if(StoreFieldEnum.STORE_AREA.getValue().equals(changeField)){
			//门店面积
			store.setConactsPhone(newValue);
		}else if(StoreFieldEnum.STORE_NAME.getValue().equals(changeField)){
			//门店名字
			store.setConactsPhone(newValue);
		}else if(StoreFieldEnum.STORE_IMAG.getValue().equals(changeField)){
			//门店图片
		}
	}

	//获取门店详情
	@RequestMapping("/store-detail")
	public HaobanResponse storeDetail(String storeId,String staffDepartmentRelatedId) {
		if(StringUtils.isAnyBlank(storeId)){
			return resultResponse(HaoBanErrCode.ERR_2);
		}
		List<StorePhotoDTO> imgDTOList = storeService.getStoreImages(storeId);
		StoreDTO store = storeService.getStore(storeId);
		StoreDetailVO vo = EntityUtil.changeEntity(StoreDetailVO.class, store);
		if(CollectionUtil.isNotEmpty(imgDTOList)){
			List<String> imgList = imgDTOList.stream().map(s->s.getImageUrl()).collect(Collectors.toList());
			vo.setImgList(imgList);
		}
		SimpleDateFormat sdf = new SimpleDateFormat("HH:mm:ss");
		if(vo.getOpenTime()!=null){
			vo.setOpenTimeStr(sdf.format(vo.getOpenTime()));
		}
		if(vo.getCloseTime()!=null){
			vo.setCloseTimeStr(sdf.format(vo.getCloseTime()));
		}
		vo.setStoreAddress(store.getFullArea()+" "+store.getStoreAddress());
		StaffDepartmentRelatedDTO staffDepartmentRelatedDTO = staffDepartmentRelatedApiService.getByStaffDepartmentRelatedId(staffDepartmentRelatedId);
		vo.setClerkCode(staffDepartmentRelatedDTO.getClerkCode());
		vo.setBindFlag(StringUtils.isEmpty(staffDepartmentRelatedDTO.getClerkCode())?0:1);
		
		return resultResponse(HaoBanErrCode.ERR_1,vo);
	}
	
	
}
